package drones.messages;

import drones.geo.Point;

import java.time.LocalDateTime;

/**
 * Represent a message on the {@link EventBus}
 *
 * @author John Ahlroos
 */
public interface Message {

    long DISPATCHER_ID = 0;

    /**
     * The recipient of the message
     */
    long targetId();

    /**
     * Represent a message with point data for the drone
     */
    record PointData(long targetId, Point point) implements Message {
    }

    /**
     * Shutdown signal for the drone. Published by the {@link drones.dispatcher.Dispatcher}
     */
    record ShutdownSignal(long targetId, long droneId, LocalDateTime time) implements Message {
    }

    /**
     * Represents the traffic condition at a tube station
     */
    record TrafficCondition(long targetId, long droneId, String tubeStation, LocalDateTime time,
                            double speed, Condition condition, double distanceToStation) implements Message {

        /**
         * The condition of the traffic
         */
        public enum Condition {
            /**
             * Heavy traffic
             */
            HEAVY,
            /**
             * Light traffic
             */
            LIGHT,
            /**
             * Moderate traffic
             */
            MODERATE
        }
    }

    /**
     * Represents a drone capacity message posted by the drone to request more waypoints
     */
    record DroneCapacity(long targetId, long droneId, int capacity) implements Message {
    }
}
